
local cql = ClassicQuestLog

ClassicQuestLogSettings = {}

cql.quests = {}

BINDING_HEADER_CLASSIC_QUEST_LOG = "Classic Quest Log"
BINDING_NAME_CLASSIC_QUEST_LOG_TOGGLE = "Show/Hide Quest Log"

function cql:OnEvent(event)
	if event=="PLAYER_LOGIN" then
		local scrollFrame = cql.scrollFrame
		scrollFrame.update = cql.UpdateLogList
		HybridScrollFrame_CreateButtons(scrollFrame, "ClassicQuestLogListTemplate")
		cql.showLevels.text:SetText("Show Levels")
		cql.showTooltips.text:SetText("Show Tooltips")
		cql.showLevels:SetChecked(ClassicQuestLogSettings.ShowLevels and true)
		cql.showTooltips:SetChecked(ClassicQuestLogSettings.ShowTooltips and true)

		cql:UpdateOverrides()
		cql:RegisterEvent("UPDATE_BINDINGS")
	elseif event=="UPDATE_BINDINGS" then
		cql:UpdateOverrides()
	elseif event=="QUEST_DETAIL" then
		cql:Hide()
	else
		if GetQuestLogSelection()==0 then
			cql:SelectFirstQuest()
		else
			cql:UpdateLogList()
		end
	end
end

function cql:OnShow()
	if WorldMapFrame:IsVisible() then
		ToggleWorldMap() -- can't have world map up at same time due to potential details frame being up
	end
	local selected = GetQuestLogSelection()
	if not selected or selected==0 then
		cql:SelectFirstQuest()
	else
		cql:SelectQuestIndex(selected)
	end
	cql:RegisterEvent("QUEST_DETAIL")
	cql:RegisterEvent("QUEST_LOG_UPDATE")
	cql:RegisterEvent("QUEST_WATCH_LIST_CHANGED")
	cql:RegisterEvent("SUPER_TRACKED_QUEST_CHANGED")
	cql:RegisterEvent("GROUP_ROSTER_UPDATE")
	cql:RegisterEvent("PARTY_MEMBER_ENABLE")
	cql:RegisterEvent("PARTY_MEMBER_DISABLE")
	cql:RegisterEvent("QUEST_POI_UPDATE")
	cql:RegisterEvent("QUEST_WATCH_UPDATE")
	cql:RegisterEvent("QUEST_ACCEPTED")
	cql:RegisterEvent("UNIT_QUEST_LOG_CHANGED")
	if not tContains(UISpecialFrames,"ClassicQuestLog") then
		tinsert(UISpecialFrames,"ClassicQuestLog")
	end
	cql.detail:ClearAllPoints()
	cql.detail:SetPoint("TOPRIGHT",-32,-63)
end

-- no need to watch these events while log isn't on screen
function cql:OnHide()
	-- only keep this window in UISpecialFrames while it's up
	for i=#UISpecialFrames,1,-1 do
		if UISpecialFrames[i]=="ClassicQuestLog" then
			tremove(UISpecialFrames,i)
		end
	end
	cql:UnregisterEvent("QUEST_DETAIL")
	cql:UnregisterEvent("QUEST_LOG_UPDATE")
	cql:UnregisterEvent("QUEST_WATCH_LIST_CHANGED")
	cql:UnregisterEvent("SUPER_TRACKED_QUEST_CHANGED")
	cql:UnregisterEvent("GROUP_ROSTER_UPDATE")
	cql:UnregisterEvent("PARTY_MEMBER_ENABLE")
	cql:UnregisterEvent("PARTY_MEMBER_DISABLE")
	cql:UnregisterEvent("QUEST_POI_UPDATE")
	cql:UnregisterEvent("QUEST_WATCH_UPDATE")
	cql:UnregisterEvent("QUEST_ACCEPTED")
	cql:UnregisterEvent("UNIT_QUEST_LOG_CHANGED")

	-- expand all headers when window hides so default doesn't lose track of collapsed quests
	local index = 1
	while index<=GetNumQuestLogEntries() do
		local _,_,_,isHeader,isCollapsed = GetQuestLogTitle(index)
		if isHeader and isCollapsed then
			ExpandQuestHeader(index)
		end
		index = index + 1
	end

end

function cql:UpdateLogList()
	local numEntries,numQuests = GetNumQuestLogEntries()
	local scrollFrame = cql.scrollFrame
	local offset = HybridScrollFrame_GetOffset(scrollFrame)
	local buttons = scrollFrame.buttons
	local selectedIndex = GetQuestLogSelection()

	cql.count.text:SetText(format("%s \124cffffffff%d/%d",QUESTS_COLON,numQuests,MAX_QUESTLOG_QUESTS))

	for i=1, #buttons do
		local index = i + offset
		local button = buttons[i]
		button.index = index
		if ( index <= numEntries ) then
			local questTitle, level, suggestedGroup, isHeader, isCollapsed, isComplete, frequency, questID = GetQuestLogTitle(index)

			button.index = index
			button.questID = questID
			button.isHeader = isHeader
			button.isCollapsed = isCollapsed

			button.normalText:SetWidth(275)
			local maxWidth = 275 -- we may shrink normalText to accomidate check and tag icons

			local color = isHeader and QuestDifficultyColors["header"] or GetQuestDifficultyColor(level)
			if not isHeader and selectedIndex==index then
				button:SetNormalFontObject("GameFontHighlight")
				button.selected:SetVertexColor(color.r,color.g,color.b)
				button.selected:Show()
			else
				button:SetNormalFontObject(color.font)
				button.selected:Hide()
			end

			if isHeader then
				button:SetText(questTitle)
				button:SetNormalTexture(isCollapsed and "Interface\\Buttons\\UI-PlusButton-Up" or "Interface\\Buttons\\UI-MinusButton-Up")
				button:SetHighlightTexture("Interface\\Buttons\\UI-PlusButton-Hilight")
				button.check:Hide()
				button.tag:Hide()
			else
				if ClassicQuestLogSettings.ShowLevels then
					button:SetText(format("  [%d] %s",level,questTitle))
				else
					button:SetText(format("  %s",questTitle))
				end
				button:SetNormalTexture("")
				button:SetHighlightTexture("")
				-- if quest is tracked, show check and shorted max normalText width
				if IsQuestWatched(index) then
					maxWidth = maxWidth - 16
					button.check:Show()
				else
					button.check:Hide()
				end
				-- display an icon to note what type of quest it is
				-- tag. daily icon can be alone or before other icons except for COMPLETED or FAILED
				local tagID
				local questTagID, tagName = GetQuestTagInfo(questID)
				if isComplete and isComplete<0 then
					tagID = "FAILED"
				elseif isComplete and isComplete>0 then
					tagID = "COMPLETED"
				elseif questTagID and questTagID==QUEST_TAG_ACCOUNT then
					local factionGroup = GetQuestFactionGroup(questID)
					if factionGroup then
						tagID = "ALLIANCE"
						if factionGroup==LE_QUEST_FACTION_HORDE then
							tagID = "HORDE"
						end
					else
						tagID = QUEST_TAG_ACCOUNT;
					end
				elseif frequency==LE_QUEST_FREQUENCY_DAILY and (not isComplete or isComplete==0) then
					tagID = "DAILY"
				elseif frequency==LE_QUEST_FREQUENCY_WEEKLY and (not isComplete or isComplete==0) then
					tagID = "WEEKLY"
				elseif questTagID then
					tagID = questTagID
				end
				button.tagID = nil
				button.tag:Hide()
				if tagID then -- this is a special type of quest
					maxWidth = maxWidth - 16
					local tagCoords = QUEST_TAG_TCOORDS[tagID]
					if tagCoords then
						button.tagID = tagID
						button.tag:SetTexCoord(unpack(tagCoords))
						button.tag:Show()
					end
				end

				-- If not a header see if any nearby group mates are on this quest
				local partyMembersOnQuest = 0
				for j=1,GetNumSubgroupMembers() do
					if IsUnitOnQuest(questIndex,"party"..j) then
						partyMembersOnQuest = partyMembersOnQuest + 1
					end
				end
				if partyMembersOnQuest>0 then
					button.groupMates:SetText("["..partyMembersOnQuest.."]")
					button.partyMembersOnQuest = partyMembersOnQuest
					button.groupMates:Show()
				else
					button.partyMembersOnQuest = nil
					button.groupMates:Hide()
				end

			end

			-- limit normalText width to the maxWidth
			button.normalText:SetWidth(min(maxWidth,button.normalText:GetStringWidth()))

			button:Show()
		else
			button:Hide()
		end
	end

	if numEntries==0 then
		cql.scrollFrame:Hide()
		cql.emptyLog:Show()
		cql.detail:Hide()
	else
		cql.scrollFrame:Show()
		cql.emptyLog:Hide()
	end

	cql:UpdateControlButtons()

	HybridScrollFrame_Update(scrollFrame, 16*numEntries, 16)
end

--[[ list entry handling ]]

function cql:ListEntryOnClick()
	local index = self.index
	if self.isHeader then
		if self.isCollapsed then
			ExpandQuestHeader(index)
		else
			CollapseQuestHeader(index)
		end
	else
		if IsModifiedClick("CHATLINK") and ChatEdit_GetActiveWindow() then
			local link = GetQuestLink(index)
			if link then
				ChatEdit_InsertLink(link)
			end
		elseif IsModifiedClick("QUESTWATCHTOGGLE") then
			cql:ToggleWatch(index)
		else
			cql:SelectQuestIndex(index)
		end
	end
	cql:UpdateLogList()
end

function cql:ToggleWatch(index)
	if not index then
		index = GetQuestLogSelection()
	end
	if index>0 then
		if IsQuestWatched(index) then -- already watched, remove from watch
			RemoveQuestWatch(index)
		else -- not watched, see if there's room to add, add if so
			if GetNumQuestWatches() >= MAX_WATCHABLE_QUESTS then
				UIErrorsFrame:AddMessage(format(QUEST_WATCH_TOO_MANY,MAX_WATCHABLE_QUESTS),1,0.1,0.1,1)
			else
				AddQuestWatch(index)
			end
		end
	end
end

-- tooltip
function cql:ListEntryOnEnter()
	if self.isHeader or not ClassicQuestLogSettings.ShowTooltips then
		return
	end

	local index = self.index

	GameTooltip:SetOwner(self,"ANCHOR_RIGHT")
	GameTooltip:AddLine((GetQuestLogTitle(index)),1,.82,0)

	-- add tag if one exists
	if self.tagID then
		local tagID, tagName = GetQuestTagInfo(self.questID)
		if tagName then
			GameTooltip:AddLine(tagName,.9,.9,.9)
		else
			tagName = self.tagID=="COMPLETED" and AUCTION_TIME_LEFT0 or _G[self.tagID]
			if tagName then
				GameTooltip:AddLine(tagName,.9,.9,.9)
			end
		end
		if QUEST_TAG_TCOORDS[self.tagID] then
			GameTooltip:AddTexture("Interface\\QuestFrame\\QuestTypeIcons", unpack(QUEST_TAG_TCOORDS[self.tagID]))
		end
	end
	-- list members on quest if they exist
	if self.partyMembersOnQuest then
		GameTooltip:AddLine(PARTY_QUEST_STATUS_ON,1,.82,0)
		for j=1,GetNumSubgroupMembers() do
			if IsUnitOnQuest(index,"party"..j) then
				GameTooltip:AddLine(GetUnitName("party"..j),.9,.9,.9)
			end
		end
		GameTooltip:AddLine(" ")
	end

	-- description
	if isComplete and isComplete>0 then
		GameTooltip:AddLine(GetQuestLogCompletionText(index),1,1,1,true)
	else
		local _, objectiveText = GetQuestLogQuestText(index)
		GameTooltip:AddLine(objectiveText,.85,.85,.85,true)
		local requiredMoney = GetQuestLogRequiredMoney(index)
		local numObjectives = GetNumQuestLeaderBoards(index)
		for i=1,numObjectives do
			local text, objectiveType, finished = GetQuestLogLeaderBoard(i,index)
			if ( text ) then
				local color = HIGHLIGHT_FONT_COLOR
				if ( finished ) then
					color = GRAY_FONT_COLOR
				end
				GameTooltip:AddLine(QUEST_DASH..text, color.r, color.g, color.b, true)
			end
		end
		if ( requiredMoney > 0 ) then
			local playerMoney = GetMoney()
			local color = HIGHLIGHT_FONT_COLOR
			if ( requiredMoney <= playerMoney ) then
				playerMoney = requiredMoney
				color = GRAY_FONT_COLOR
			end
			GameTooltip:AddLine(QUEST_DASH..GetMoneyString(playerMoney).." / "..GetMoneyString(requiredMoney), color.r, color.g, color.b);
		end

	end


	GameTooltip:Show()
end

--[[ selection ]]

function cql:SelectQuestIndex(index)

	SelectQuestLogEntry(index)

	StaticPopup_Hide("ABANDON_QUEST")
	StaticPopup_Hide("ABANDON_QUEST_WITH_ITEMS")
	SetAbandonQuest()

	if ( questIndex == 0 ) then
		cql.selectedIndex = nil
		ClassicQuestLogDetailScrollFrame:Hide()
	elseif index>0 and index<=GetNumQuestLogEntries() and not select(4,GetQuestLogTitle(index)) then
		ClassicQuestLogDetailScrollFrame:Show()
		QuestInfo_Display(QUEST_TEMPLATE_LOG, ClassicQuestLogDetailScrollChildFrame)
		ClassicQuestLogDetailScrollFrameScrollBar:SetValue(0) -- scroll to top
	end

	cql:UpdateLogList()
end

-- selects the first quest in the log (if any)
function cql:SelectFirstQuest()
	for i=1,GetNumQuestLogEntries() do
		if not select(4,GetQuestLogTitle(i)) then
			cql:SelectQuestIndex(i)
			return
		end
	end
	cql:SelectQuestIndex(0) -- if we reached here, select nothing
end

--[[ control buttons ]]

function cql:UpdateControlButtons()
	local selectionIndex = GetQuestLogSelection()
	if selectionIndex==0 then
		cql.abandon:Disable()
		cql.push:Disable()
		cql.track:Disable()
	else
		local questID = select(8,GetQuestLogTitle(selectionIndex))
		cql.abandon:SetEnabled(GetAbandonQuestName() and CanAbandonQuest(questID))
		cql.push:SetEnabled(GetQuestLogPushable() and IsInGroup())
		cql.track:Enable()
	end
end

--[[ map button ]]

function cql:ShowMap()
	cql:Hide() -- can't let map quest details fight with our details
	local selectionIndex = GetQuestLogSelection()
	if selectionIndex==0 then
		ToggleWorldMap()
	else
		local questID = select(8,GetQuestLogTitle(selectionIndex))
		if not WorldMapFrame:IsVisible() then
			ToggleWorldMap()
		end
		QuestMapFrame_ShowQuestDetails(questID)
	end
end

--[[ overrides ]]

-- if a user sets a key in Key Bindings -> AddOns -> Classic Quest Log, then
-- we leave the default binding and micro button alone.
-- if no key is set, we override the default's binding and hook the macro button

function cql:UpdateOverrides()
	local key = GetBindingKey("CLASSIC_QUEST_LOG_TOGGLE")
	if key and cql.overridingKey then
		ClearOverrideBindings(cql)
		cql.overridingKey = nil
	else -- there's no binding for addon, so override the default stuff
		-- hook the ToggleQuestLog (if it's not been hooked before)
		if not cql.oldToggleQuestLog then
			cql.oldToggleQuestLog = ToggleQuestLog
			function ToggleQuestLog(...)
				if cql.overridingKey then
					cql:SetShown(not cql:IsVisible()) -- to toggle our window if overriding
					return
				else
					return cql.oldToggleQuestLog(...) -- and default stuff if they clear overriding
				end
			end
		end
		-- now see if default toggle quest binding has changed
		local newKey = GetBindingKey("TOGGLEQUESTLOG")
		if cql.overridingKey~=newKey and newKey then
			ClearOverrideBindings(cql)
			SetOverrideBinding(cql,false,newKey,"CLASSIC_QUEST_LOG_TOGGLE")
			cql.overridingKey = newKey
		end
	end
end
